import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CreateTaskComponent } from './create-task.component';
import { ReactiveFormsModule } from '@angular/forms';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { ToastrService } from 'ngx-toastr';
import { of, throwError } from 'rxjs';

describe('CreateTaskComponent', () => {
  let component: CreateTaskComponent;
  let fixture: ComponentFixture<CreateTaskComponent>;
  let httpMock: HttpTestingController;
  let toastrService: jasmine.SpyObj<ToastrService>;

  beforeEach(async () => {
    const toastrSpy = jasmine.createSpyObj('ToastrService', ['success', 'error']);
    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule, HttpClientTestingModule],
      declarations: [CreateTaskComponent],
      providers: [
        { provide: MatDialogRef, useValue: { close: jasmine.createSpy('close') } },
        { provide: MAT_DIALOG_DATA, useValue: {} },
        { provide: ToastrService, useValue: toastrSpy },
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(CreateTaskComponent);
    component = fixture.componentInstance;
    httpMock = TestBed.inject(HttpTestingController);
    toastrService = TestBed.inject(ToastrService) as jasmine.SpyObj<ToastrService>;

    fixture.detectChanges();
  });

  afterEach(() => {
    httpMock.verify();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should initialize the form with default values', () => {
    expect(component.taskForm).toBeTruthy();
    expect(component.taskForm.get('commandGroup')?.value).toBe('');
    expect(component.taskForm.get('extraCommands')?.value).toEqual([]);
    expect(component.taskForm.get('date')?.value).toBe('');
    expect(component.taskForm.get('time')?.value).toBe('');
    expect(component.taskForm.get('notes')?.value).toBe('');
  });

  it('should load command groups and set availableCommandGroups', () => {
    const mockCommandGroups = { 'hydra:member': [{ uuid: '1', name: 'Group 1' }, { uuid: '2', name: 'Group 2' }] };
    
    component.loadCommandGroups();

    const req = httpMock.expectOne(`${component.baseUrl}/command-groups`);
    expect(req.request.method).toBe('GET');
    req.flush(mockCommandGroups);

    expect(component.availableCommandGroups.length).toBe(2);
    expect(component.availableCommandGroups).toEqual(mockCommandGroups['hydra:member']);
  });

  it('should handle error when loading command groups', () => {
    component.loadCommandGroups();

    const req = httpMock.expectOne(`${component.baseUrl}/command-groups`);
    req.flush('Error loading command groups', { status: 500, statusText: 'Server Error' });

    expect(toastrService.error).toHaveBeenCalledWith('Error al cargar los grupos de comandos');
  });

  it('should load individual commands and set availableIndividualCommands', () => {
    const mockCommands = { 'hydra:member': [{ uuid: '1', name: 'Command 1' }, { uuid: '2', name: 'Command 2' }] };

    component.loadIndividualCommands();

    const req = httpMock.expectOne(`${component.baseUrl}/commands`);
    expect(req.request.method).toBe('GET');
    req.flush(mockCommands);

    expect(component.availableIndividualCommands.length).toBe(2);
    expect(component.availableIndividualCommands).toEqual(mockCommands['hydra:member']);
  });

  it('should handle error when loading individual commands', () => {
    component.loadIndividualCommands();

    const req = httpMock.expectOne(`${component.baseUrl}/commands`);
    req.flush('Error loading individual commands', { status: 500, statusText: 'Server Error' });

    expect(toastrService.error).toHaveBeenCalledWith('Error al cargar los comandos individuales');
  });

  it('should fetch selected group commands on command group change', () => {
    component.taskForm.patchValue({ commandGroup: '1' });

    component.onCommandGroupChange();

    const mockGroupCommands = { commands: [{ uuid: '1', name: 'Group Command 1' }, { uuid: '2', name: 'Group Command 2' }] };
    
    const req = httpMock.expectOne(`${component.baseUrl}/command-groups/1`);
    expect(req.request.method).toBe('GET');
    req.flush(mockGroupCommands);

    expect(component.selectedGroupCommands.length).toBe(2);
    expect(component.selectedGroupCommands).toEqual(mockGroupCommands.commands);
  });

  it('should handle error when fetching group commands', () => {
    component.taskForm.patchValue({ commandGroup: '1' });
    component.onCommandGroupChange();

    const req = httpMock.expectOne(`${component.baseUrl}/command-groups/1`);
    req.flush('Error loading group commands', { status: 500, statusText: 'Server Error' });

    expect(toastrService.error).toHaveBeenCalledWith('Error al cargar los comandos del grupo seleccionado');
  });

  it('should save the task successfully', () => {
    const mockFormData = {
      commandGroup: '1',
      extraCommands: ['1', '2'],
      date: '2024-09-27',
      time: '12:00',
      notes: 'Test notes'
    };

    component.taskForm.setValue(mockFormData);

    component.saveTask();

    const expectedPayload = {
      commandGroups: ['/command-groups/1'],
      commands: ['/commands/1', '/commands/2'],
      dateTime: jasmine.any(String),
      notes: 'Test notes'
    };

    const req = httpMock.expectOne(component.apiUrl);
    expect(req.request.method).toBe('POST');
    req.flush({}); // Mock successful response

    expect(toastrService.success).toHaveBeenCalledWith('Tarea creada con éxito');
    expect(component.dialogRef.close).toHaveBeenCalledWith(true);
  });

  it('should not save the task if form is invalid', () => {
    component.taskForm.setValue({
      commandGroup: '',
      extraCommands: [],
      date: '',
      time: '',
      notes: ''
    });

    component.saveTask();

    expect(toastrService.error).toHaveBeenCalledWith('Por favor, rellene todos los campos obligatorios');
    expect(component.dialogRef.close).not.toHaveBeenCalled();
  });

  it('should handle error when saving the task', () => {
    const mockFormData = {
      commandGroup: '1',
      extraCommands: ['1'],
      date: '2024-09-27',
      time: '12:00',
      notes: 'Test notes'
    };

    component.taskForm.setValue(mockFormData);
    component.saveTask();

    const req = httpMock.expectOne(component.apiUrl);
    expect(req.request.method).toBe('POST');
    req.flush('Error creating task', { status: 500, statusText: 'Server Error' });

    expect(toastrService.error).toHaveBeenCalledWith('Error al crear la tarea');
  });

  it('should close the dialog', () => {
    component.close();
    expect(component.dialogRef.close).toHaveBeenCalled();
  });
});
