import { Component } from '@angular/core';
import {PageEvent} from "@angular/material/paginator";
import {DatePipe} from "@angular/common";
import {MatTableDataSource} from "@angular/material/table";
import {MatDialog} from "@angular/material/dialog";
import {ToastrService} from "ngx-toastr";
import {HttpClient} from "@angular/common/http";
import {DataService} from "./data.service";
import {EditClientComponent} from "../../shared/clients/edit-client/edit-client.component";
import {CreateClientComponent} from "../../shared/clients/create-client/create-client.component";
import {DeleteModalComponent} from "../../../../shared/delete_modal/delete-modal/delete-modal.component";
import { throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
@Component({
  selector: 'app-client-tab-view',
  templateUrl: './client-tab-view.component.html',
  styleUrl: './client-tab-view.component.css'
})
export class ClientTabViewComponent {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  dataSource = new MatTableDataSource<any>();
  length: number = 0;
  loading:boolean = false;
  itemsPerPage: number = 10;
  pageSizeOptions: number[] = [5, 10, 25, 100];
  page: number = 1;
  filters: { [key: string]: string } = {};
  organizationalUnits: any[] = [];
  datePipe: DatePipe = new DatePipe('es-ES');

  private apiUrl = `${this.baseUrl}/clients`;

  columns = [
    {
      columnDef: 'id',
      header: 'ID',
      cell: (client: any) => `${client.id}`
    },
    {
      columnDef: 'name',
      header: 'Nombre del cliente',
      cell: (client: any) => `${client.name}`
    },
    {
      columnDef: 'ip',
      header: 'IP',
      cell: (client: any) => `${client.ip}`
    },
    {
      columnDef: 'mac',
      header: 'Mac',
      cell: (client: any) => `${client.mac}`
    },
    {
      columnDef: 'organizationalUnit',
      header: 'Pertenece a',
      cell: (client: any) => `${client.organizationalUnit.name}`
    },
    {
      columnDef: 'createdAt',
      header: 'Fecha de creación',
      cell: (client: any) => `${this.datePipe.transform(client.createdAt, 'dd/MM/yyyy hh:mm:ss')}`
    }
  ];
  displayedColumns = [...this.columns.map(column => column.columnDef), 'actions'];

  constructor(
    private dataService: DataService,
    public dialog: MatDialog,
    private toastService: ToastrService,
    private http: HttpClient
  ) {}

  ngOnInit(): void {
    this.getClients();
    this.loadOrganizationalUnits();
  }

  getClients() {
    this.http.get<any>(`${this.apiUrl}?&page=${this.page}&itemsPerPage=${this.itemsPerPage}`, { params: this.filters }).subscribe(
      (data) => {
        this.dataSource.data = data['hydra:member'];
        this.length = data['hydra:totalItems'];
      },
      (error) => {
        console.error('Error fetching commands', error);
      }
    );
  }

  onEditClick(event: MouseEvent, uuid: string): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(EditClientComponent, { data: { uuid }, width: '900px' } );
    dialogRef.afterClosed().subscribe(() => this.getClients());
  }

  addClient(event: MouseEvent, organizationalUnit:any = null): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(CreateClientComponent, { data: { organizationalUnit }, width: '900px'});
    dialogRef.afterClosed().subscribe(() => {
      this.getClients();
    });
  }

  onDeleteClick(event: MouseEvent, client: any): void {
    event.stopPropagation();
    const dialogRef = this.dialog.open(DeleteModalComponent, {
      width: '400px'
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.http.delete<void>(`${this.apiUrl}/${client.uuid}`).pipe(
          catchError(error => {
            this.toastService.error(error.error['hydra:description']);
            return throwError(error);
          })
        ).subscribe(() => {
          this.toastService.success('Elemento eliminado correctamente');
          this.getClients();
        });
      }
    });
  }

  resetFilters() {
    this.loading = true;
    this.filters = {};
    this.getClients();
  }

  loadOrganizationalUnits() {
    this.loading = true;
    this.http.get<any>( `${this.baseUrl}/organizational-units?&page=1&itemsPerPage=10000`).subscribe(
      response => {
        this.organizationalUnits = response['hydra:member'];
        this.loading = false;
      },
      error => {
        console.error('Error fetching parent units:', error);
        this.loading = false;
      }
    );
  }

  search(): void {
    this.loading = true;
    this.getClients()
  }

  onPageChange(event: any): void {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
    this.length = event.length;
    this.getClients();
  }
}
