import { Component, OnInit, Output, EventEmitter, Inject } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { ToastrService } from 'ngx-toastr';
import { DataService } from '../../../services/data.service';

@Component({
  selector: 'app-create-organizational-unit',
  templateUrl: './create-organizational-unit.component.html',
  styleUrls: ['./create-organizational-unit.component.css']
})
export class CreateOrganizationalUnitComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  isLinear = true;
  generalFormGroup: FormGroup;
  additionalInfoFormGroup: FormGroup;
  networkSettingsFormGroup: FormGroup;
  classroomInfoFormGroup: FormGroup;
  types: string[] = ['organizational-unit', 'classrooms-group', 'classroom', 'clients-group'];
  typeTranslations: { [key: string]: string } = {
    'organizational-unit': 'Unidad organizativa',
    'classrooms-group': 'Grupo de aulas',
    'classroom': 'Aula',
    'clients-group': 'Grupo de clientes'
  };
  protected p2pModeOptions = [
    { name: 'Leecher', value: 'p2p-mode-leecher' },
    { name: 'Peer', value: 'p2p-mode-peer' },
    { name: 'Seeder', value: 'p2p-mode-seeder' },
  ];
  multicastModeOptions: { name: string, value: string }[] = [
    { name: 'Modo 1', value: 'mode1' },
    { name: 'Modo 2', value: 'mode2' },
    { name: 'Modo 3', value: 'mode3' },
  ];
  parentUnits: any[] = [];
  hardwareProfiles: any[] = [];
  calendars: any[] = [];
  selectedCalendarUuid: string | null = null;

  @Output() unitAdded = new EventEmitter();

  constructor(
    private _formBuilder: FormBuilder,
    private dialogRef: MatDialogRef<CreateOrganizationalUnitComponent>,
    private http: HttpClient,
    private toastService: ToastrService,
    private dataService: DataService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {
    this.generalFormGroup = this._formBuilder.group({
      name: ['', Validators.required],
      parent: [this.data.parent ? this.data.parent['@id'] : null],
      description: [''],
      type: ['', Validators.required]
    });
    this.additionalInfoFormGroup = this._formBuilder.group({
      comments: [''],
    });
    this.networkSettingsFormGroup = this._formBuilder.group({
      nextServer: [''],
      bootFileName: [''],
      proxy: [''],
      dns: [''],
      netmask: [''],
      router: [''],
      ntp: [''],
      p2pMode: [''],
      p2pTime: [0, Validators.min(0)],
      mcastIp: [''],
      mcastSpeed: [0, Validators.min(0)],
      mcastPort: [0, Validators.min(0)],
      mcastMode: [''],
      menu: [''],
      hardwareProfile: [''],
      validation: [false],
      remoteCalendar: [''] 
    });
    this.classroomInfoFormGroup = this._formBuilder.group({
      location: [''],
      projector: [false],
      board: [false],
      capacity: [0, Validators.min(0)]
    });
  }

  ngOnInit() {
    this.loadParentUnits();
    this.loadHardwareProfiles();
    this.loadCalendars();
  }

  loadParentUnits() {
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=1000`;
    this.http.get<any>(url).subscribe(
      response => this.parentUnits = response['hydra:member'],
      error => console.error('Error fetching parent units:', error)
    );
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => this.hardwareProfiles = data,
      error => console.error('Error fetching hardware profiles', error)
    );
  }

  loadCalendars() {
    const apiUrl = `${this.baseUrl}/remote-calendars?page=1&itemsPerPage=30`;
    this.http.get<any>(apiUrl).subscribe(
      response => this.calendars = response['hydra:member'],
      error => {
        console.error('Error loading calendars', error);
        this.openSnackBar(true, 'Error loading calendars');
      }
    );
  }

  private cleanFormValues(formGroup: FormGroup): any {
    const cleanedValues: any = {};
    Object.keys(formGroup.controls).forEach(key => {
      const value = formGroup.get(key)?.value;
      if (value !== '' && value !== 0) {
        cleanedValues[key] = value;
      }
    });
    return cleanedValues;
  }

  onSubmit() {
    if (this.isFormValid()) {
      const formData: any = this.buildPayload();
      const postUrl = `${this.baseUrl}/organizational-units`;
      const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

      this.http.post<any>(postUrl, formData, { headers }).subscribe(
        response => {
          this.unitAdded.emit();
          this.dialogRef.close();
          this.openSnackBar(false, 'Cliente creado exitosamente');
        },
        error => {
          console.error('Error al realizar POST:', error);
          this.openSnackBar(true, 'Error al crear la unidad organizativa: ' + error.error['hydra:description']);
        }
      );
    }
  }

  private isFormValid(): boolean {
    return this.generalFormGroup.valid && 
           this.additionalInfoFormGroup.valid && 
           this.networkSettingsFormGroup.valid && 
           (this.generalFormGroup.value.type !== 'classroom' || this.classroomInfoFormGroup.valid);
  }

  private buildPayload(): any {
    const generalFormValues = this.cleanFormValues(this.generalFormGroup);
    const additionalInfoFormValues = this.cleanFormValues(this.additionalInfoFormGroup);
    const networkSettingsFormValues = this.cleanFormValues(this.networkSettingsFormGroup);
    const classroomInfoFormValues = this.cleanFormValues(this.classroomInfoFormGroup);

    return {
      ...generalFormValues,
      ...classroomInfoFormValues,
      comments: additionalInfoFormValues.comments,
      networkSettings: { ...networkSettingsFormValues },
      remoteCalendar: this.selectedCalendarUuid ? `/remote-calendars/${this.selectedCalendarUuid}` : null,
      menu: networkSettingsFormValues.menu || null,
      hardwareProfile: networkSettingsFormValues.hardwareProfile || null,
    };
  }

  onCalendarChange(event: any) {
    this.selectedCalendarUuid = event.value;
  }

  onNoClick(): void {
    this.dialogRef.close();
  }

  openSnackBar(isError: boolean, message: string) {
    if (isError) {
      this.toastService.error('Error al crear el cliente: ' + message, 'Error');
    } else {
      this.toastService.success('Cliente creado exitosamente', 'Éxito');
    }
  }
}
