import { Component } from '@angular/core';
import {MatTableDataSource} from "@angular/material/table";
import {MatDialog} from "@angular/material/dialog";
import {HttpClient} from "@angular/common/http";
import { CreatePxeTemplateComponent } from '../pxe/create-pxeTemplate/create-pxe-template.component';
import {PageEvent} from "@angular/material/paginator";
import {ToastrService} from "ngx-toastr";
import {DatePipe} from "@angular/common";
import {DataService} from "./data.service";

@Component({
  selector: 'app-pxe-boot-files',
  templateUrl: './pxe-boot-files.component.html',
  styleUrl: './pxe-boot-files.component.css'
})
export class PxeBootFilesComponent {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  pxeTemplates: any[] = []; 
  currentPage: number = 1;
  dataSource = new MatTableDataSource<any>();
  length: number = 0;
  alertMessage: string | null = null;
  itemsPerPage: number = 10;
  loading:boolean = false;
  filters: { [key: string]: string } = {};
  page: number = 1;
  pageSizeOptions: number[] = [5, 10, 20, 40, 100];
  datePipe: DatePipe = new DatePipe('es-ES');
  selectedElements: string[] = [];
  templateOptions: any[] = [];
  columns = [
    {
      columnDef: 'id',
      header: 'ID',
      cell: (user: any) => `${user.id}`
    },
    {
      columnDef: 'templateName',
      header: 'Nombre de la plantilla',
      cell: (user: any) => `${user.template.name}`
    },
    {
      columnDef: 'clients',
      header: 'Clientes',
      cell: (user: any) => user.clients
    },
    {
      columnDef: 'createdAt',
      header: 'Fecha de creación',
      cell: (user: any) => `${this.datePipe.transform(user.createdAt, 'dd/MM/yyyy hh:mm:ss')}`
    }
  ];
  displayedColumns = [...this.columns.map(column => column.columnDef), 'actions'];

  private apiUrl = `${this.baseUrl}/pxe-boot-files`;

  constructor(
    public dialog: MatDialog,
    private http: HttpClient,
    private toastService: ToastrService,
    private dataService: DataService
  )
  { }

  ngOnInit(): void {
    this.search();
    this.loadAlert();
    this.dataService.getPxeTemplateOptions().subscribe(
      data => {
        this.templateOptions = data;
      },
      error => {
        console.error('Error fetching template options', error);
      }
    );
  }

  search(): void {
    this.loading = true;
    this.dataService.getPxeBootFiles(this.filters).subscribe(
      data => {
        this.dataSource.data = data;
        this.loading = false;
      },
      error => {
        console.error('Error fetching pxe boot files', error);
        this.loading = false;
      }
    );
  }

  toggleAction(image: any, action:string): void {
    switch (action) {
      case 'create':
        this.http.post(`${this.apiUrl}/server/${image.uuid}/post`, {}).subscribe({
          next: () => {
            console.log('Plantilla cambiada');
            this.search();
          },
          error: (error) => {
            console.error('Error al cambiar la imagen:', error);
          }
        });
        break;
      case 'delete':
        this.http.post(`${this.apiUrl}/server/${image.uuid}/delete`, {}).subscribe({
          next: () => {
            console.log('Plantilla cambiada');
            this.search();
          },
          error: (error) => {
            console.error('Error al cambiar la imagen:', error);
          }
        });
        break;
      default:
        console.error('Acción no soportada:', action);
        break;
    }
  }

  deletePxeTemplate(uuid: string) {
    // Lógica para eliminar una plantilla
  }

  editPxeTemplate(template: any) {
    const dialogRef = this.dialog.open(CreatePxeTemplateComponent, {
      data: template
    });

    dialogRef.afterClosed().subscribe(() => {
      this.search();
    });
  }

  /* editPxeBootFile(bootFile: any) {
    const dialogRef = this.dialog.open(CreatePxeBootFileComponent, {
      data: {
        clients: bootFile.clients,
        bootFile: bootFile
      }
    });
  
    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.loadPxeBootFiles(); // Refrescar la lista después de la edición
      }
    });
  } */

  applyFilter() {
    this.http.get<any>(`${this.apiUrl}?page=${this.page}&itemsPerPage=${this.itemsPerPage}`).subscribe({
      next: (response) => {
        this.dataSource.data = response['hydra:member'];
        this.length = response['hydra:totalItems'];
      },
      error: (error) => {
        console.error('Error al cargar las imágenes:', error);
      }
    });
  }

  onPageChange(event: PageEvent) {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
    this.applyFilter();
  }

  loadAlert() {
    this.http.get(`${this.apiUrl}/server/get-collection`)
      .subscribe(response => {
        // @ts-ignore
        this.alertMessage = response.templates.length
      }, error => {
        console.error('Error al cargar la información del alert', error);
      });
  }

  getIcon(): { name: string, color: string } {
    if (Number(this.alertMessage) === this.length) {
      return { name: 'check_circle', color: 'green' }; // Icono de check verde
    } else {
      return { name: 'cancel', color: 'red' }; // Icono de cruz roja
    }
  }

  syncOgCore(): void {
    this.http.post(`${this.apiUrl}/sync`, {})
      .subscribe(response => {
        this.toastService.success('Sincronización completada');
        this.search()
      }, error => {
        console.error('Error al sincronizar', error);
        this.toastService.error('Error al sincronizar');
      });
  }

}
