import { Component, OnInit, ViewChild } from '@angular/core';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { CreateSubnetComponent } from './create-subnet/create-subnet.component';
import { MatDialog } from '@angular/material/dialog';
import { HttpClient } from '@angular/common/http';
import { DeleteModalComponent } from '../../../shared/delete_modal/delete-modal/delete-modal.component';
import { ToastrService } from 'ngx-toastr';
import { AddClientsToSubnetComponent } from './add-clients-to-subnet/add-clients-to-subnet.component';

export interface Subnet {
  '@id': string;
  '@type': string;
  name: string;
  netmask: string;
  ipAddress: string;
  nextServer: string;
  bootFileName: string;
  createdAt: string;
  createdBy: string;
  uuid: string;
  id: number;
}

@Component({
  selector: 'app-og-dhcp-subnets',
  templateUrl: './og-dhcp-subnets.component.html',
  styleUrls: ['./og-dhcp-subnets.component.css']
})
export class OgDhcpSubnetsComponent  {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  displayedColumns: string[] = ['name', 'netmask', 'ipAddress', 'nextServer', 'bootFileName', 'actions'];
  dataSource = new MatTableDataSource<Subnet>([]);
  length = 0;
  itemsPerPage: number = 10;
  page = 0;
  pageSizeOptions: number[] = [5, 10, 20];
  alertMessage: string | null = null;

  @ViewChild(MatPaginator) paginator: MatPaginator | undefined;
  
  columns = [
    { columnDef: 'name', header: 'Name', cell: (subnet: Subnet) => subnet.name },
    { columnDef: 'netmask', header: 'Netmask', cell: (subnet: Subnet) => subnet.netmask },
    { columnDef: 'ipAddress', header: 'IP Address', cell: (subnet: Subnet) => subnet.ipAddress },
    { columnDef: 'nextServer', header: 'Next Server', cell: (subnet: Subnet) => subnet.nextServer },
    { columnDef: 'bootFileName', header: 'Boot File Name', cell: (subnet: Subnet) => subnet.bootFileName },
  ];

  constructor(public dialog: MatDialog, private http: HttpClient, private toastService: ToastrService) {}

  ngOnInit() {
    this.loadSubnets();
  }

  loadSubnets() {
    this.http.get<any>(`${this.baseUrl}/subnets?page=1&itemsPerPage=${this.itemsPerPage}`).subscribe({
      next: (response) => {
        this.dataSource.data = response['hydra:member'];
        this.length = response['hydra:totalItems'];
      },
      error: error => {
        console.error('Error al cargar plantillas PXE:', error);
      }
    });

    if (this.paginator) {
      this.dataSource.paginator = this.paginator;
    }
  }

  syncSubnets() {
    console.log('Sincronizando subnets...');
  }

  addSubnet() {
    console.log('Añadiendo nueva subnet...');
    const dialogRef = this.dialog.open(CreateSubnetComponent, {
      width: '400px'
    });

    dialogRef.afterClosed().subscribe(result => {
      console.log('The dialog was closed');
      this.loadSubnets()
    });
  }

  editSubnet(subnet: Subnet) {
    console.log('Editando subnet:', subnet);
    const dialogRef = this.dialog.open(CreateSubnetComponent, {
      width: '400px',
      data: { subnet } 
    });
  
    dialogRef.afterClosed().subscribe(result => {
      console.log('The dialog was closed');
      this.loadSubnets()
    });
  }

  deleteSubnet(subnet: Subnet): void {
    console.log(subnet);
    const dialogRef = this.dialog.open(DeleteModalComponent, {
      width: '300px',
      data: { name: subnet.name }
    });
  
    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        const apiUrl = `${this.baseUrl}${subnet['@id']}`;
        
        this.http.delete(apiUrl).subscribe({
          next: () => {
            console.log('Subnet deleted successfully');
            this.dataSource.data = this.dataSource.data.filter(s => s !== subnet);
            this.length = this.dataSource.data.length;
            this.toastService.success('Subnet deleted successfully');
          },
          error: (error) => {
            console.error('Error deleting subnet:', error);
          }
        });
      } else {
        console.log('Subnet deletion cancelled');
      }
    });
  }

  updateSubnet() {
    console.log('Update Subnet action selected');
  }

  addClientsToSubnet(subnet: Subnet) {
    
    const dialogRef = this.dialog.open(AddClientsToSubnetComponent, {
      width: '600px',
      data: { subnetUuid: subnet.uuid, subnetName: subnet.name }  
    });
  
    dialogRef.afterClosed().subscribe(result => {
      console.log('The dialog was closed');
      this.loadSubnets();
    });
  }

  onPageChange(event: any) {
    this.page = event.pageIndex;
    this.itemsPerPage = event.pageSize;
  }

  getIcon() {
    return { name: 'info', color: 'blue' };
  }
}
